"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const data_handler_1 = require("./data-handler");
class DataManager {
    static _registerHandler(domain, dataHandler, map, type) {
        if (map.has(domain)) {
            console.error(`${type}-handler is already registered at domain "${domain}"`.red);
        }
        else {
            map.set(domain, dataHandler);
            console.error(`${type}-handler was registered at domain "${domain}"`.yellow);
        }
    }
    static get instance() {
        if (!this._instance) {
            this._instance = new DataManager();
        }
        return this._instance;
    }
    constructor() {
        this._dataGetters = new Map();
        this._dataSetters = new Map();
    }
    getTable(entries) {
        if (!entries) {
            console.log('no entries'.red, entries);
            return [];
        }
        const builder = new TableBuilder(entries);
        this.buildValue(0, builder.craateRow(), builder);
        builder.printWarnings();
        return builder.rows;
    }
    makeRequest(requests) {
        return undefined;
    }
    registerHandler(domain, dataHandler) {
        if (data_handler_1.isDataSetter(dataHandler)) {
            DataManager._registerHandler(domain, dataHandler, this._dataSetters, data_handler_1.dataSetterType);
        }
        if (data_handler_1.isDataGetter(dataHandler)) {
            DataManager._registerHandler(domain, dataHandler, this._dataGetters, data_handler_1.dataGetterType);
        }
        if (data_handler_1.isDataAdder(dataHandler)) {
            DataManager._registerHandler(domain, dataHandler, this._dataAdders, data_handler_1.dataAdderType);
        }
        if (data_handler_1.isDataRemover(dataHandler)) {
            DataManager._registerHandler(domain, dataHandler, this._dataRemovers, data_handler_1.dataRemoverType);
        }
    }
    buildValue(index, row, builder) {
        if (index >= builder.entries.length) {
            return;
        }
        const entry = builder.entries[index];
        if (!entry) {
            this.buildValue(index + 1, row, builder);
            return;
        }
        const getter = this._dataGetters.get(entry.domain);
        if (!getter) {
            builder.addMissingDomains(entry.domain);
            this.buildValue(index + 1, row, builder);
            return;
        }
        const options = {};
        if (entry.options.values) {
            for (const property in entry.options.values) {
                options[property] = entry.options.values[property];
            }
        }
        if (entry.options.dependencies) {
            for (const property in entry.options.dependencies) {
                options[property] = row[entry.options.dependencies[property]];
            }
        }
        const response = getter.getData(options);
        if (!response) {
            console.log(`get-domain "${entry.domain}" returned null response`);
            this.buildValue(index + 1, row, builder);
            return;
        }
        const values = response.value;
        if (!values || !values.length) {
            this.buildValue(index + 1, row, builder);
            return;
        }
        let counter = 0;
        for (const value of values) {
            if (typeof value === 'undefined') {
                continue;
            }
            let currentRow;
            if (counter === 0) {
                currentRow = row;
            }
            else {
                currentRow = builder.craateRow(row);
            }
            counter++;
            currentRow[entry.id] = value;
            this.buildValue(index + 1, currentRow, builder);
        }
    }
}
exports.DataManager = DataManager;
class TableBuilder {
    constructor(entries) {
        this._entries = entries;
        this._rows = [];
        this._missingDomains = [];
    }
    get entries() {
        return this._entries;
    }
    get rows() {
        const rows = [];
        const undefinedMap = new Map();
        for (const _row of this._rows) {
            const row = [];
            for (const entry of this._entries) {
                if (!entry) {
                    row.push(undefined);
                }
                else {
                    let value = _row[entry.id];
                    row.push(value);
                    if (typeof value === 'undefined') {
                        if (undefinedMap.has(entry.id)) {
                            const counter = undefinedMap.get(entry.id);
                            undefinedMap.set(entry.id, counter + 1);
                        }
                        else {
                            undefinedMap.set(entry.id, 1);
                        }
                    }
                }
            }
            rows.push(row);
        }
        const undefinedArray = [];
        undefinedMap.forEach((value, key) => {
            undefinedArray.push(`${key}(${value})`);
        });
        if (undefinedArray.length) {
            console.log('undefined values'.red, undefinedArray);
        }
        return rows;
    }
    addMissingDomains(domain) {
        if (!this._missingDomains.includes(domain)) {
            this._missingDomains.push(domain);
        }
    }
    craateRow(row = {}) {
        const result = {};
        for (const property in row) {
            result[property] = row[property];
        }
        this._rows.push(result);
        return result;
    }
    printWarnings() {
        let entriesEntries = false;
        for (const entry of this._entries) {
            if (!entry) {
                entriesEntries = true;
                break;
            }
        }
        if (entriesEntries) {
            console.log('invalid entries'.red, this._entries);
        }
        if (this._missingDomains.length) {
            console.log('get-domains dont exist'.red, this._missingDomains);
        }
    }
}
//# sourceMappingURL=data-manager.js.map